@@ -28,7 +28,7 @@ import os
 import io
 import shutil
 
-from firewall.config import ETC_FIREWALLD
+from firewall import config
 from firewall.functions import checkIP, checkIP6, checkIPnMask, \
     checkIP6nMask, u2b_if_py2, check_mac, check_port, checkInterface, \
     checkProtocol
@@ -320,6 +320,8 @@ class ipset_ContentHandler(IO_Object_ContentHandler):
         self.item.parser_check_element_attrs(name, attrs)
         if name == "ipset":
             if "type" in attrs:
+                if attrs["type"] not in IPSET_TYPES:
+                    raise FirewallError(errors.INVALID_TYPE, "%s" % attrs["type"])
                 self.item.type = attrs["type"]
             if "version" in attrs:
                 self.item.version = attrs["version"]
@@ -382,15 +384,17 @@ def ipset_reader(filename, path):
     ipset.check_name(ipset.name)
     ipset.filename = filename
     ipset.path = path
-    ipset.builtin = False if path.startswith(ETC_FIREWALLD) else True
+    ipset.builtin = False if path.startswith(config.ETC_FIREWALLD) else True
     ipset.default = ipset.builtin
     handler = ipset_ContentHandler(ipset)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
     name = "%s/%s" % (path, filename)
-    with open(name, "r") as f:
+    with open(name, "rb") as f:
+        source = sax.InputSource(None)
+        source.setByteStream(f)
         try:
-            parser.parse(f)
+            parser.parse(source)
         except sax.SAXParseException as msg:
             raise FirewallError(errors.INVALID_IPSET,
                                 "not a valid ipset file: %s" % \
@@ -439,9 +443,9 @@ def ipset_writer(ipset, path=None):
             log.error("Backup of file '%s' failed: %s", name, msg)
 
     dirpath = os.path.dirname(name)
-    if dirpath.startswith(ETC_FIREWALLD) and not os.path.exists(dirpath):
-        if not os.path.exists(ETC_FIREWALLD):
-            os.mkdir(ETC_FIREWALLD, 0o750)
+    if dirpath.startswith(config.ETC_FIREWALLD) and not os.path.exists(dirpath):
+        if not os.path.exists(config.ETC_FIREWALLD):
+            os.mkdir(config.ETC_FIREWALLD, 0o750)
         os.mkdir(dirpath, 0o750)
 
     f = io.open(name, mode='wt', encoding='UTF-8')
